﻿using BlacksmithWorkshopContracts.BindingModels;
using BlacksmithWorkshopContracts.BusinessLogicContracts;
using BlacksmithWorkshopContracts.BusinessLogicsContracts;
using BlacksmithWorkshopContracts.SearchModels;
using Microsoft.Extensions.Logging;
using System.Reflection;

namespace BlacksmithWorkshopView
{
	public partial class FormStore : Form
	{
		private readonly ILogger _logger;
		private readonly IStoreLogic _logicS;
		private readonly IComponentLogic _logicC;
		private int? _id;
		public int Id { set { _id = value; } }
		public FormStore(ILogger<FormComponent> logger, IStoreLogic logicS, IComponentLogic logicC)
		{
			InitializeComponent();
			_logger = logger;
			_logicS = logicS;
			_logicC = logicC;
		}
		private void FormStore_Load(object sender, EventArgs e)
		{
			dataGridView.Columns["ColumnManufactureName"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
			if (_id.HasValue)
			{
				try
				{
					_logger.LogInformation("Получение магазина");
					var view = _logicS.ReadElement(new StoreSearchModel
					{
						Id = _id.Value
					});
					if (view != null)
					{
						textBoxName.Text = view.StoreName;
						textBoxAddress.Text = view.Address;
						dateTimePicker.Value = view.OpeningDate;
					}
					LoadData();
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "Ошибка получения магазина");
					MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
					MessageBoxIcon.Error);
				}
			}
		}
		private void LoadData()
		{
			if (_id == null)
			{
				return;
			}
			try
			{
				var list = _logicS.ReadElement(new StoreSearchModel
				{
					Id = _id.Value
				})?.Manufactures;
				if (list != null)
				{
					dataGridView.Rows.Clear();
					foreach (var el in list)
					{
						dataGridView.Rows.Add(new object[] { el.Item1.ManufactureName, el.Item1.Price, el.Item2 });
					}
				}
				_logger.LogInformation("Загрузка компонентов");
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка загрузки компонентов");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}
		private void ButtonSave_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(textBoxName.Text))
			{
				MessageBox.Show("Заполните название", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if (string.IsNullOrEmpty(textBoxAddress.Text))
			{
				MessageBox.Show("Заполните адрес", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			_logger.LogInformation("Сохранение магазина");
			try
			{
				var model = new StoreBindingModel
				{
					Id = _id ?? 0,
					StoreName = textBoxName.Text,
					Address = textBoxAddress.Text,
					OpeningDate = dateTimePicker.Value
				};
				var operationResult = _id.HasValue ? _logicS.Update(model) : _logicS.Create(model);
				if (!operationResult)
				{
					throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
				}
				MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
				DialogResult = DialogResult.OK;
				Close();
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка сохранения магазина");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}
		private void ButtonCancel_Click(object sender, EventArgs e)
		{
			DialogResult = DialogResult.Cancel;
			Close();
		}
	}
}
