﻿using BlacksmithWorkShopBusinessLogic.BusinessLogics;
using BlacksmithWorkshopContracts.BindingModels;
using BlacksmithWorkshopContracts.BusinessLogicsContracts;
using BlacksmithWorkshopContracts.SearchModels;
using BlacksmithWorkshopContracts.StoragesContracts;
using BlacksmithWorkshopContracts.ViewModels;
using BlacksmithWorkshopDataModels.Models;
using Microsoft.Extensions.Logging;
using System.Security.Cryptography.X509Certificates;

namespace BlacksmithWorkshopBusinessLogic.BusinessLogics
{
	public class StoreLogic : IStoreLogic
	{
		private readonly ILogger _logger;
		private readonly IStoreStorage _storeStorage;
		public StoreLogic(ILogger<ComponentLogic> logger, IStoreStorage storeStorage)
		{
			_logger = logger;
			_storeStorage = storeStorage;
		}
		public List<StoreViewModel>? ReadList(StoreSearchModel? model)
		{
			_logger.LogInformation("Readlist. StoreName: {Storename}. Id: {Id}", model?.StoreName, model?.Id);
			var list = model == null ? _storeStorage.GetFullList() : _storeStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("Readlist. Count: {Count}", list.Count);
			return list;
		}
		public StoreViewModel? ReadElement(StoreSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. StoreName: {StoreName}. Id: {Id}", model.StoreName, model.Id);
			var element = _storeStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement find. Id: {Id}", element.Id);
			return element;
		}
		public bool Create(StoreBindingModel model)
		{
			CheckModel(model);
			if (_storeStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
		}
		public bool Update(StoreBindingModel model)
		{
			CheckModel(model);
			if (_storeStorage.Update(model) == null)
			{
				_logger.LogWarning("Update operation failed");
				return false;
			}
			return true;
		}
		public bool Delete(StoreBindingModel model)
		{
			CheckModel(model, false);
			_logger.LogInformation("Delete. Id:{Id}", model.Id);
			if (_storeStorage.Delete(model) == null)
			{
				_logger.LogWarning("Delete operation failed");
				return false;
			}
			return true;
		}
		private void SaveChanges(StoreViewModel? model)
		{
			if (model == null)
			{
				return;
			}
			_storeStorage.Update(new StoreBindingModel()
			{
				StoreName = model.StoreName,
				Address = model.Address,
				OpeningDate = model.OpeningDate,
				Manufactures = model.Manufactures
			});
		}
		public bool AddManufacture(IManufactureModel manufacture, StoreSearchModel model, int count)
		{
			if (model == null)
			{
				throw new ArgumentNullException("Невозможно найти магазин", nameof(model));
			}
			if (count <= 0)
			{
				throw new ArgumentException("Количество добавляемых изделий должно быть больше нуля");
			}
			_logger.LogInformation("Adding manufacture {manufacture} in store {store}. Id = {id}", manufacture.ManufactureName, model.StoreName, model.Id);
			//ищем магазин
			StoreViewModel? selectedStore = _storeStorage.GetElement(model);
			if (selectedStore == null)
			{
				_logger.LogWarning("Store with id {Id} not found", model.Id);
				return false;
			}
			//ищем изделие в магазине
			foreach (var man in selectedStore.Manufactures)
			{
				if (man.Item1.Id == manufacture.Id)//если изделие уже есть в магазине, увеличиваем его количество
				{
					selectedStore.Manufactures[manufacture.Id - 1] = (manufacture, selectedStore.Manufactures[manufacture.Id - 1].Item2 + count);
					SaveChanges(selectedStore);
					_logger.LogInformation("Filled store Id = {storeId} with manufacture Id = {manufactureId}. Count = {count}", model.Id, manufacture.Id, count);
					return true;
				}
			}
			//иначе добавляем изделие в магазин в указанном количестве
			selectedStore.Manufactures.Add((manufacture, count));
			SaveChanges(selectedStore);
			_logger.LogInformation("Added new manufacture Id = {manufactureId} in store Id = {storeId}. Count = {count}", manufacture.Id, model.Id, count);
			return true;
		}
		private void CheckModel(StoreBindingModel model, bool withParams = true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			if (string.IsNullOrEmpty(model.StoreName))
			{
				throw new ArgumentNullException("Нет названия магазина", nameof(model.StoreName));
			}
			if (string.IsNullOrEmpty(model.Address))
			{
				throw new ArgumentNullException("Нет адреса магазина", nameof(model.StoreName));
			}
			_logger.LogInformation("Store. StoreName: {StoreName}. Address: {Address}. Id: {Id}", model.StoreName, model.Address, model.Id);
			var element = _storeStorage.GetElement(new StoreSearchModel//проверка на уникальность
			{
				StoreName = model.StoreName
			});
			if (element != null && element.Id != model.Id)
			{
				throw new InvalidOperationException("Магазин с таким названием уже есть");
			}
		}
	}
}